/*  Copyright (C) 2015 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/24/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __cxx_fs_error_h__
#define __cxx_fs_error_h__ 1

/*   std includes   */
#include <system_error>
#include <memory>

/*   project headers   */
#include <cxx/filesystem/fs-path.h>



namespace duct {
  namespace filesystem {
    
    class filesystem_error : public std::system_error
    {
      /*   message     */
      struct message {
        path path1, path2; // may be empty()
        std::string  what;  // not built until needed
      };
      
      /*   empty path  */
      static const path _M_empty_path;
      
    public:
      /*    constructors   */
      filesystem_error(const std::string & what_arg, std::error_code ec) noexcept
      : std::system_error(ec, what_arg)
      {
        try {
          _M_ptr.reset(new message);
        } catch(...) { _M_ptr.reset(); }
      }
      
      filesystem_error(const std::string & what_arg, const path& path1, std::error_code ec) noexcept
      : std::system_error(ec, what_arg)
      {
        try {
          _M_ptr.reset(new message);
          _M_ptr->path1 = path1;
        } catch (...) { _M_ptr.reset(); }
      }
      
      filesystem_error(const std::string & what_arg, const path& path1, const path& path2, std::error_code ec) noexcept
      : std::system_error(ec, what_arg)
      {
        try {
          _M_ptr.reset(new message);
          _M_ptr->path1 = path1;
          _M_ptr->path2 = path2;
        }
        catch (...) { _M_ptr.reset(); }
      }
            
      /*   get messages   */
      const path& path1() const noexcept {
        return _M_ptr ? _M_ptr->path1 : _M_empty_path;
      }
      
      const path& path2() const noexcept {
        return _M_ptr ? _M_ptr->path2 : _M_empty_path;
      }
      
      const char* what() const noexcept;
      
    private:
      /*   message in shared pinter   */
      std::shared_ptr<message> _M_ptr;
    };
    
  }
}

#endif
