/*  Copyright (C) 2015 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/24/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */
#include <cassert>
#include <sys/stat.h>
#include <unistd.h>

/*   project headers  */
#include "fs-status.h"
#include "fs-error.h"
#include "fs-posix-api.h"


namespace duct {
  namespace filesystem {
    
    namespace api = __posix_api;
    
    
    path read_symlink(const path& p, std::error_code& ec) noexcept
    {
      int errval = 0;
      auto retval = api::read_symlink(p, errval);
      ec = std::error_code(errval, std::system_category());
      
      return retval;
    }
    
     
    void last_write_time(const path& p, const file_time_type new_time, std::error_code& ec) noexcept
    {
      int errval = 0;
      api::last_write_time(p, new_time, errval);
      ec = std::error_code(errval, std::system_category());
    }

    
  }  //  namespace filesystem
}  //  namespace duct
