/*  Copyright (C) 2012-2013 Zoltn Nagy
 *  Created by Zoltn Nagy on 16/2/12.
 *  This file created by D. Soper 5 August 2015
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

//   std includes
#include <cmath>
#include <limits>
#include <complex>

//   project headers
#include <proc-epaglue.h>


namespace duct {
  
  void epaglue::generate(random_engine_type& random, shower_history<epa>& hist)
  {
    std::uniform_real_distribution<double> rng(0.0,1.0);
    
    double phi = (2.0*M_PI)*rng(random);
    double c = 2.0*rng(random)-1.0;              // cos(theta)
    double s = std::sqrt(1.0-c*c), E = 0.5*_M_Ecm;  // sin(theta)
    // Note that we select a quark flavor, but then we don't use it.
    
    /*    x and y components  */
    double px = E*s*std::cos(phi), py = E*s*std::sin(phi);
        
    /*    resizing the state    */
    auto hard = hist.injecting_a_hard_stage();
    
    hard->state.resize(0,2);
    hard->state.set_model(_M_mdl);

    //----- flavours -----
    hard->state[-1].flavor =  11;
    hard->state[ 0].flavor = -11;
    hard->state[ 1].flavor = 0;
    hard->state[ 2].flavor = 0;
    
    //----- momenta -----
    hard->state[-1].momentum = lorentzvector<double>(0.0,0.0,-E, E);
    hard->state[ 0].momentum = lorentzvector<double>(0.0,0.0, E, E);
    hard->state[ 1].momentum = lorentzvector<double>( px,  py,  E*c, E);
    hard->state[ 2].momentum = lorentzvector<double>(-px, -py, -E*c, E);
    
    
    /*    color connections    */
    hard->state.insert_closed_string({1,2});   // For a singlet state of two gluons.
    hard->state.insert_closed_stringP({1,2});  // For a singlet state of two gluons.
    
    /*    color suppression of the state   */
    hard->state.color_suppression_index = 0U;
   
    /*     weight    */
    hard->weights.unitary.main = 1.0;
    
    /*    starting scale of the shower    */
    hard->scale = sqr(_M_Ecm);
    
    /*   vetos    */
    hard->vetos = shower_vetos{};
  }
  
  
}


